@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2003, 2004 Red Hat, Inc.
+ * Copyright (C) 2003, 2004, 2005, 2006 Red Hat, Inc.
  * Copyright (C) 2003 Keith Packard
  * 
  * This program is free software; you can redistribute it and/or
@@ -53,16 +53,16 @@
 typedef struct
 {
   /* top of stack is first in list */
-  GList *compositor_windows;
+  GList *compositor_nodes;
   WsWindow *glw;
   int idle_id;
 } ScreenInfo;
 
 struct MetaCompositor
 {
-  MetaDisplay *display;
+  MetaDisplay *meta_display;
   
-  Ws *ws;
+  WsDisplay *display;
   
   GHashTable *window_hash;
   
@@ -83,9 +83,9 @@ struct MetaCompositor
 static void
 free_window_hash_value (void *v)
 {
-  DrawableNode *drawable_node = v;
+  CmDrawableNode *drawable_node = v;
   
-  drawable_node_unref (drawable_node);
+  g_object_unref (G_OBJECT (drawable_node));
 }
 #endif /* HAVE_COMPOSITE_EXTENSIONS */
 
@@ -97,12 +97,12 @@ meta_compositor_new (MetaDisplay *display)
   
   compositor = g_new0 (MetaCompositor, 1);
   
-  compositor->ws = ws_new (NULL);
+  compositor->display = ws_display_new (NULL);
   
-  ws_init_test (compositor->ws);
-  ws_set_ignore_grabs (compositor->ws, TRUE);
+  ws_display_init_test (compositor->display);
+  ws_display_set_ignore_grabs (compositor->display, TRUE);
   
-  compositor->display = display;
+  compositor->meta_display = display;
   
   compositor->window_hash = g_hash_table_new_full (meta_unsigned_long_hash,
 						   meta_unsigned_long_equal,
@@ -161,7 +161,7 @@ static void
 draw_windows (MetaScreen *screen,
 	      GList      *list)
 {
-  Node *node;
+  CmNode *node;
   
   if (!list)
     return;
@@ -169,12 +169,17 @@ draw_windows (MetaScreen *screen,
   node = list->data;
   
   draw_windows (screen, list->next);
-  node->render (node);
+
+#if 0
+  g_print ("rendering: %p\n", node);
+#endif
+  
+  cm_node_render (node);
 }
 
 static MetaScreen *
 node_get_screen (Display *dpy,
-		 DrawableNode *node)
+		 CmDrawableNode *node)
 {
   /* FIXME: we should probably have a reverse mapping
    * from nodes to screens
@@ -186,25 +191,25 @@ node_get_screen (Display *dpy,
 
 static void
 handle_restacking (MetaCompositor *compositor,
-		   DrawableNode *node,
-		   DrawableNode *above)
+		   CmDrawableNode *node,
+		   CmDrawableNode *above)
 {
   GList *window_link, *above_link;
   MetaScreen *screen;
   ScreenInfo *scr_info;
   
-  screen = node_get_screen (compositor->display->xdisplay, node);
+  screen = node_get_screen (compositor->meta_display->xdisplay, node);
   scr_info = screen->compositor_data;
   
-  window_link = g_list_find (scr_info->compositor_windows, node);
-  above_link  = g_list_find (scr_info->compositor_windows, above);
+  window_link = g_list_find (scr_info->compositor_nodes, node);
+  above_link  = g_list_find (scr_info->compositor_nodes, above);
   
   if (!window_link || !above_link)
     return;
   
   if (window_link == above_link)
     {
-      /* This can happen if the topmost window is raise above
+      /* This can happen if the topmost window is raised above
        * the GL window
        */
       return;
@@ -214,10 +219,10 @@ handle_restacking (MetaCompositor *compositor,
     {
       ScreenInfo *scr_info = screen->compositor_data;
       
-      scr_info->compositor_windows =
-	g_list_delete_link (scr_info->compositor_windows, window_link);
-      scr_info->compositor_windows =
-	g_list_insert_before (scr_info->compositor_windows, above_link, node);
+      scr_info->compositor_nodes =
+	g_list_delete_link (scr_info->compositor_nodes, window_link);
+      scr_info->compositor_nodes =
+	g_list_insert_before (scr_info->compositor_nodes, above_link, node);
     }
 }
 
@@ -226,23 +231,24 @@ process_configure_notify (MetaCompositor  *compositor,
                           XConfigureEvent *event)
 {
   WsWindow *above_window;
-  DrawableNode *node = g_hash_table_lookup (compositor->window_hash,
+  CmDrawableNode *node = g_hash_table_lookup (compositor->window_hash,
 					    &event->window);
-  DrawableNode *above_node;
+  CmDrawableNode *above_node;
   MetaScreen *screen;
   ScreenInfo *scr_info;
   
   if (!node)
     return;
   
-  screen = node_get_screen (compositor->display->xdisplay, node);
+  screen = node_get_screen (compositor->meta_display->xdisplay, node);
   scr_info = screen->compositor_data;
   
-  above_window = ws_window_lookup (node->drawable->ws, event->above);
+  above_window = ws_window_lookup (WS_RESOURCE (node->drawable)->display,
+				   event->above);
   
   if (above_window == scr_info->glw)
     {
-      above_node = scr_info->compositor_windows->data;
+      above_node = scr_info->compositor_nodes->data;
     }
   else
     {
@@ -270,11 +276,15 @@ static void
 process_map (MetaCompositor     *compositor,
              XMapEvent          *event)
 {
-  DrawableNode *node;
+  CmDrawableNode *node;
   MetaScreen *screen;
+
+  /* FIXME: do we sometimes get mapnotifies for windows that are
+   * not (direct) children of the root?
+   */
   
   /* See if window was mapped as child of root */
-  screen = meta_display_screen_for_root (compositor->display,
+  screen = meta_display_screen_for_root (compositor->meta_display,
 					 event->event);
   
   if (screen == NULL)
@@ -291,12 +301,12 @@ process_map (MetaCompositor     *compositor,
     {
       XWindowAttributes attrs;
       
-      meta_error_trap_push_with_return (compositor->display);
+      meta_error_trap_push_with_return (compositor->meta_display);
       
-      XGetWindowAttributes (compositor->display->xdisplay,
+      XGetWindowAttributes (compositor->meta_display->xdisplay,
 			    event->window, &attrs);
       
-      if (meta_error_trap_pop_with_return (compositor->display, TRUE) != Success)
+      if (meta_error_trap_pop_with_return (compositor->meta_display, TRUE) != Success)
         {
 	  meta_topic (META_DEBUG_COMPOSITOR, "Failed to get attributes for window 0x%lx\n",
 		      event->window);
@@ -309,7 +319,7 @@ process_map (MetaCompositor     *compositor,
     }
   else
     {
-      drawable_node_set_viewable (node, TRUE);
+      cm_drawable_node_set_viewable (node, TRUE);
     }
   
   /* We don't actually need to invalidate anything, because we will
@@ -324,11 +334,11 @@ static void
 process_unmap (MetaCompositor     *compositor,
                XUnmapEvent        *event)
 {
-  DrawableNode *node;
+  CmDrawableNode *node;
   MetaScreen *screen;
   
   /* See if window was unmapped as child of root */
-  screen = meta_display_screen_for_root (compositor->display,
+  screen = meta_display_screen_for_root (compositor->meta_display,
 					 event->event);
   
   if (screen == NULL)
@@ -343,7 +353,7 @@ process_unmap (MetaCompositor     *compositor,
 			      &event->window);
   if (node != NULL)
     {
-      drawable_node_set_viewable (node, FALSE);
+      cm_drawable_node_set_viewable (node, FALSE);
     }
 }
 #endif /* HAVE_COMPOSITE_EXTENSIONS */
@@ -356,7 +366,7 @@ process_create (MetaCompositor     *compositor,
   MetaScreen *screen;
   XWindowAttributes attrs;
   
-  screen = meta_display_screen_for_root (compositor->display,
+  screen = meta_display_screen_for_root (compositor->meta_display,
 					 event->parent);
   
   if (screen == NULL)
@@ -367,12 +377,12 @@ process_create (MetaCompositor     *compositor,
       return;
     }
   
-  meta_error_trap_push_with_return (compositor->display);
+  meta_error_trap_push_with_return (compositor->meta_display);
   
-  XGetWindowAttributes (compositor->display->xdisplay,
+  XGetWindowAttributes (compositor->meta_display->xdisplay,
 			event->window, &attrs);
   
-  if (meta_error_trap_pop_with_return (compositor->display, TRUE) != Success)
+  if (meta_error_trap_pop_with_return (compositor->meta_display, TRUE) != Success)
     {
       meta_topic (META_DEBUG_COMPOSITOR, "Failed to get attributes for window 0x%lx\n",
 		  event->window);
@@ -395,7 +405,7 @@ process_destroy (MetaCompositor      *compositor,
 {
   MetaScreen *screen;
   
-  screen = meta_display_screen_for_root (compositor->display,
+  screen = meta_display_screen_for_root (compositor->meta_display,
 					 event->event);
   
   if (screen == NULL)
@@ -423,10 +433,10 @@ process_reparent (MetaCompositor      *compositor,
    */
   MetaScreen *event_screen;
   MetaScreen *parent_screen;
-  DrawableNode *node;
+  CmDrawableNode *node;
   XWindowAttributes attrs;
   
-  event_screen = meta_display_screen_for_root (compositor->display,
+  event_screen = meta_display_screen_for_root (compositor->meta_display,
 					       event->event);
   
   if (event_screen == NULL)
@@ -441,7 +451,7 @@ process_reparent (MetaCompositor      *compositor,
 	      "Reparent window 0x%lx new parent 0x%lx received on 0x%lx\n",
 	      event->window, event->parent, event->event);
   
-  parent_screen = meta_display_screen_for_root (compositor->display,
+  parent_screen = meta_display_screen_for_root (compositor->meta_display,
 						event->parent);
   
   if (parent_screen == NULL)
@@ -456,12 +466,12 @@ process_reparent (MetaCompositor      *compositor,
   node = g_hash_table_lookup (compositor->window_hash,
 			      &event->window);
   
-  meta_error_trap_push_with_return (compositor->display);
+  meta_error_trap_push_with_return (compositor->meta_display);
   
-  XGetWindowAttributes (compositor->display->xdisplay,
+  XGetWindowAttributes (compositor->meta_display->xdisplay,
 			event->window, &attrs);
   
-  if (meta_error_trap_pop_with_return (compositor->display, TRUE) != Success)
+  if (meta_error_trap_pop_with_return (compositor->meta_display, TRUE) != Success)
     {
       meta_topic (META_DEBUG_COMPOSITOR, "Failed to get attributes for window 0x%lx\n",
 		  event->window);
@@ -556,8 +566,6 @@ update (gpointer data)
   glLoadIdentity ();
   gluOrtho2D (0, 1.0, 0.0, 1.0);
   
-  ws_window_raise (gl_window);
-  
   glClearColor (0.0, 0.5, 0.5, 0.0);
   glClear (GL_COLOR_BUFFER_BIT);
   
@@ -574,31 +582,46 @@ update (gpointer data)
   
   glEnd ();
   
-  glEnable (GL_TEXTURE_2D);
-  draw_windows (screen, scr_info->compositor_windows);
+  ws_window_raise (gl_window);
   
-  /* FIXME: we should probably grab the server around the raise/swap */
+  glEnable (GL_TEXTURE_2D);
+  draw_windows (screen, scr_info->compositor_nodes);
+  
+  /* FIXME: we should probably grab the server around the raise/swap
+   */
+
+#if 0
+  ws_display_grab (ws_drawable_get_display ((WsDrawable *)gl_window));
+#endif
   
   ws_window_gl_swap_buffers (gl_window);
   glFinish();
 
+#if 0
+  ws_display_ungrab (ws_drawable_get_display ((WsDrawable *)gl_window));
+#endif
+  
   scr_info->idle_id = 0;
 
-  g_print ("returning FALSE\n");
-  
   return FALSE;
 }
 
 static void
-do_repaint (DrawableNode *node, gpointer data)
+queue_repaint (CmDrawableNode *node, gpointer data)
 {
     MetaScreen *screen = data;
     ScreenInfo *scr_info = screen->compositor_data;
 
-    g_print ("do repaint\n");
+#if 0
+    g_print ("queueing repaint for %p\n", node);
+#endif
     
     if (!scr_info->idle_id)
+    {
+	g_print ("paint\n");
+  
 	scr_info->idle_id = g_idle_add (update, screen);
+    }
 }
 #endif /* HAVE_COMPOSITE_EXTENSIONS */
 
@@ -611,7 +634,7 @@ meta_compositor_add_window (MetaCompositor    *compositor,
                             XWindowAttributes *attrs)
 {
 #ifdef HAVE_COMPOSITE_EXTENSIONS
-  DrawableNode *node;
+  CmDrawableNode *node;
   MetaScreen *screen;
   WsDrawable *drawable;
   ScreenInfo *scr_info;
@@ -632,7 +655,7 @@ meta_compositor_add_window (MetaCompositor    *compositor,
       return;
     }
   
-  drawable = (WsDrawable *)ws_window_lookup (compositor->ws, xwindow);
+  drawable = (WsDrawable *)ws_window_lookup (compositor->display, xwindow);
 
   scr_info = screen->compositor_data;
 
@@ -645,9 +668,9 @@ meta_compositor_add_window (MetaCompositor    *compositor,
     }
   else
     {
-      node = drawable_node_new (drawable);
+      node = cm_drawable_node_new (drawable);
 
-      drawable_node_set_damage_func (node, do_repaint, screen);
+      cm_drawable_node_set_damage_func (node, queue_repaint, screen);
       
 #if 0
       drawable_node_set_deformation_func (node, wavy, NULL);
@@ -656,13 +679,13 @@ meta_compositor_add_window (MetaCompositor    *compositor,
   
   /* FIXME: we should probably just store xid's directly */
   g_hash_table_insert (compositor->window_hash,
-		       &(node->drawable->xid), node);
+		       &(WS_RESOURCE (node->drawable)->xid), node);
   
   /* assume cwindow is at the top of the stack as it was either just
    * created or just reparented to the root window
    */
-  scr_info->compositor_windows = g_list_prepend (scr_info->compositor_windows,
-						 node);
+  scr_info->compositor_nodes = g_list_prepend (scr_info->compositor_nodes,
+					       node);
   
 #endif /* HAVE_COMPOSITE_EXTENSIONS */
 }
@@ -672,7 +695,7 @@ meta_compositor_remove_window (MetaCompositor    *compositor,
                                Window             xwindow)
 {
 #ifdef HAVE_COMPOSITE_EXTENSIONS
-  DrawableNode *node;
+  CmDrawableNode *node;
   MetaScreen *screen;
   ScreenInfo *scr_info;
   
@@ -689,10 +712,10 @@ meta_compositor_remove_window (MetaCompositor    *compositor,
       return;
     }
   
-  screen = node_get_screen (compositor->display->xdisplay, node);
+  screen = node_get_screen (compositor->meta_display->xdisplay, node);
   scr_info = screen->compositor_data;
   
-  scr_info->compositor_windows = g_list_remove (scr_info->compositor_windows,
+  scr_info->compositor_nodes = g_list_remove (scr_info->compositor_nodes,
 					      node);
   
   /* Frees node as side effect */
@@ -710,32 +733,32 @@ meta_compositor_manage_screen (MetaCompositor *compositor,
   ScreenInfo *scr_info = g_new0 (ScreenInfo, 1);
   
   WsScreen *ws_screen =
-      ws_screen_get_from_number (compositor->ws, screen->number);
+      ws_display_get_screen_from_number (compositor->display, screen->number);
   WsWindow *root = ws_screen_get_root_window (ws_screen);
   WsRegion *region;
   
   scr_info->glw = ws_window_new_gl (root);
-  scr_info->compositor_windows = NULL;
+  scr_info->compositor_nodes = NULL;
   scr_info->idle_id = 0;
 
   g_print ("setting compositor_data for screen %p to %p\n", screen, scr_info);
   screen->compositor_data = scr_info;
   
-  ws_init_composite (compositor->ws);
-  ws_init_damage (compositor->ws);
-  ws_init_fixes (compositor->ws);
+  ws_display_init_composite (compositor->display);
+  ws_display_init_damage (compositor->display);
+  ws_display_init_fixes (compositor->display);
   
   ws_window_redirect_subwindows (root);
   ws_window_set_override_redirect (scr_info->glw, TRUE);
   ws_window_unredirect (scr_info->glw);
   
-  region = ws_region_new (compositor->ws);
+  region = ws_region_new (compositor->display);
   ws_window_set_input_shape (scr_info->glw, region);
-  ws_region_unref (region);
+  g_object_unref (G_OBJECT (region));
   
   ws_window_map (scr_info->glw);
   
-  ws_sync (compositor->ws);
+  ws_display_sync (compositor->display);
   
 #endif
 }
@@ -750,11 +773,12 @@ meta_compositor_unmanage_screen (MetaCompositor *compositor,
   if (!compositor->enabled)
     return; /* no extension */
   
-  while (scr_info->compositor_windows != NULL)
+  while (scr_info->compositor_nodes != NULL)
     {
-      DrawableNode *node = scr_info->compositor_windows->data;
+      CmDrawableNode *node = scr_info->compositor_nodes->data;
       
-      meta_compositor_remove_window (compositor, node->drawable->xid);
+      meta_compositor_remove_window (compositor,
+				     WS_RESOURCE (node->drawable)->xid);
     }
   /* FIXME: free scr_info */
   
@@ -762,12 +786,12 @@ meta_compositor_unmanage_screen (MetaCompositor *compositor,
 }
 
 #ifdef HAVE_COMPOSITE_EXTENSIONS  
-static DrawableNode *
+static CmDrawableNode *
 window_to_node (MetaCompositor *compositor,
 		MetaWindow *window)
 {
   Window xwindow;
-  DrawableNode *node;
+  CmDrawableNode *node;
   
   if (window->frame)
     xwindow = window->frame->xwindow;
@@ -791,13 +815,14 @@ typedef struct
 typedef struct
 {
   MetaWindow *window;
-  DrawableNode *node;
+  CmDrawableNode *node;
   
   DoubleRect start;
   DoubleRect target;
   
   double start_time;
   int idle_id;
+    int repaint_id;
 
   MetaMinimizeFinishedFunc finished_func;
   gpointer		   finished_data;
@@ -814,11 +839,13 @@ stop_minimize (gpointer data)
 {
   MiniInfo *info = data;
   
-  drawable_node_set_deformation_func (info->node, NULL, NULL);
+  g_source_remove (info->repaint_id);
   
+  cm_drawable_node_set_deformation_func (info->node, NULL, NULL);
+
   if (info->finished_func)
     info->finished_func (info->finished_data);
-  
+
   g_free (info);
   
   return FALSE;
@@ -845,7 +872,7 @@ minimize_deformation (gdouble time,
   
   *out_x = interpolate (pos, in_x, info->target.x + info->target.width * ((in_x - info->start.x)  / info->start.width), 10 * in_y);
   *out_y = interpolate (pos, in_y, info->target.y + info->target.height * ((in_y - info->start.y)  / info->start.height), 1.0);
-  
+
   if (elapsed > MINIMIZE_TIME)
     {
       g_assert (info->node);
@@ -854,6 +881,16 @@ minimize_deformation (gdouble time,
     }
 }
 
+static gboolean
+do_minimize_animation (gpointer data)
+{
+    MiniInfo *info = data;
+    
+    queue_repaint (info->node,
+		   node_get_screen (info->window->display->xdisplay,
+				    info->node));
+}
+
 static void
 convert (MetaScreen *screen,
 	 int x, int y, int width, int height,
@@ -878,7 +915,7 @@ meta_compositor_minimize (MetaCompositor           *compositor,
 {
 #ifdef HAVE_COMPOSITE_EXTENSIONS
   MiniInfo *info = g_new (MiniInfo, 1);
-  DrawableNode *node = window_to_node (compositor, window);
+  CmDrawableNode *node = window_to_node (compositor, window);
   WsRectangle start;
   MetaScreen *screen = window->screen;
   
@@ -902,6 +939,9 @@ meta_compositor_minimize (MetaCompositor           *compositor,
   info->finished_func = finished;
   info->finished_data = data;
   
-  drawable_node_set_deformation_func (node, minimize_deformation, info);
+  cm_drawable_node_set_deformation_func (node, minimize_deformation, info);
+
+  info->repaint_id = g_idle_add (do_minimize_animation, info);
+  
 #endif
 }
