@@ -78,7 +78,7 @@ static guint changed_idle;
 #endif
 static GList *listeners = NULL;
 
-static gboolean use_system_font = TRUE;
+static gboolean use_system_font = FALSE;
 static PangoFontDescription *titlebar_font = NULL;
 static MetaVirtualModifier mouse_button_mods = Mod1Mask;
 static MetaFocusMode focus_mode = META_FOCUS_MODE_CLICK;
@@ -86,19 +86,19 @@ static gboolean raise_on_click = TRUE;
 static char* current_theme = NULL;
 static int num_workspaces = 4;
 static MetaActionDoubleClickTitlebar action_double_click_titlebar =
-    META_ACTION_DOUBLE_CLICK_TITLEBAR_TOGGLE_SHADE;
+    META_ACTION_DOUBLE_CLICK_TITLEBAR_TOGGLE_MAXIMIZE;
 static gboolean application_based = FALSE;
 static gboolean disable_workarounds = FALSE;
 static gboolean auto_raise = FALSE;
 static gboolean auto_raise_delay = 500;
-static gboolean provide_visual_bell = TRUE;
+static gboolean provide_visual_bell = FALSE;
 static gboolean bell_is_audible = TRUE;
 static gboolean reduced_resources = FALSE;
 static gboolean gnome_accessibility = FALSE;
 static char *cursor_theme = NULL;
 static int   cursor_size = 24;
 
-static MetaVisualBellType visual_bell_type = META_VISUAL_BELL_INVALID;
+static MetaVisualBellType visual_bell_type = META_VISUAL_BELL_FULLSCREEN_FLASH;
 static MetaButtonLayout button_layout = {
   {
     META_BUTTON_FUNCTION_MENU,
@@ -122,7 +122,7 @@ static char *terminal_command = NULL;
 static char *workspace_names[MAX_REASONABLE_WORKSPACES] = { NULL, };
 
 #ifdef HAVE_GCONF
-static gboolean update_use_system_font   (gboolean    value);
+static gboolean update_use_system_font    (gboolean    value);
 static gboolean update_titlebar_font      (const char *value);
 static gboolean update_mouse_button_mods  (const char *value);
 static gboolean update_focus_mode         (const char *value);
@@ -141,8 +141,6 @@ static gboolean update_window_binding     (const char *name,
                                            const char *value);
 static gboolean update_screen_binding     (const char *name,
                                            const char *value);
-static gboolean update_binding            (MetaKeyPref *binding,
-                                           const char  *value);
 static gboolean update_command            (const char  *name,
                                            const char  *value);
 static gboolean update_terminal_command   (const char *value);
@@ -162,6 +160,8 @@ static char* gconf_key_for_workspace_name (int i);
 
 static void queue_changed (MetaPreference  pref);
 #endif /* HAVE_GCONF */
+static gboolean update_binding            (MetaKeyPref *binding,
+                                           const char  *value);
 
 static void     init_bindings             (void);
 static void     init_commands             (void);
@@ -301,6 +301,29 @@ cleanup_error (GError **error)
       *error = NULL;
     }
 }
+
+/* get_bool returns TRUE if *val is filled in, FALSE otherwise */
+static gboolean
+get_bool (const char *key, gboolean *val)
+{
+  GError     *err = NULL;
+  GConfValue *value;
+  gboolean    filled_in = FALSE;
+
+  value = gconf_client_get (default_client, key, &err);
+  cleanup_error (&err);
+  if (value)
+    {
+      if (value->type == GCONF_VALUE_BOOL)
+        {
+          *val = gconf_value_get_bool (value);
+          filled_in = TRUE;
+        }
+      gconf_value_free (value);
+    }
+
+  return filled_in;
+}
 #endif /* HAVE_GCONF */
 
 void
@@ -350,10 +373,8 @@ meta_prefs_init (void)
   update_focus_mode (str_val);
   g_free (str_val);  
 
-  bool_val = gconf_client_get_bool (default_client, KEY_RAISE_ON_CLICK,
-				    &err);
-  cleanup_error (&err);
-  update_raise_on_click (bool_val);
+  if (get_bool (KEY_RAISE_ON_CLICK, &bool_val))
+    update_raise_on_click (bool_val);
   
   str_val = gconf_client_get_string (default_client,
                                      KEY_ACTION_DOUBLE_CLICK_TITLEBAR,
@@ -362,11 +383,15 @@ meta_prefs_init (void)
   update_action_double_click_titlebar (str_val);
   g_free (str_val);
 
-  bool_val = gconf_client_get_bool (default_client, KEY_AUTO_RAISE,
-				    &err);
-  cleanup_error (&err);
-  update_auto_raise (bool_val);
+  if (get_bool (KEY_AUTO_RAISE, &bool_val))
+    update_auto_raise (bool_val);
 
+  /* FIXME: Since auto_raise_delay of 0 is valid and gconf_client_get_int
+   * silently returns that value when KEY_AUTO_RAISE_DELAY doesn't exist,
+   * we should be using gconf_client_get() instead if we cared about
+   * careful error checking of the key-doesn't-exist case.  Since this
+   * setting is crap, though, I'm adding a FIXME instead of fixing it.  ;-)
+   */
   int_val = gconf_client_get_int (default_client, KEY_AUTO_RAISE_DELAY,
 				  &err);
   cleanup_error (&err);
@@ -384,10 +409,8 @@ meta_prefs_init (void)
    * just lazy. But they keys ought to be set, anyhow.
    */
   
-  bool_val = gconf_client_get_bool (default_client, KEY_USE_SYSTEM_FONT,
-                                    &err);
-  cleanup_error (&err);
-  update_use_system_font (bool_val);  
+  if (get_bool (KEY_USE_SYSTEM_FONT, &bool_val))
+    update_use_system_font (bool_val);
   
   str_val = gconf_client_get_string (default_client, KEY_TITLEBAR_FONT,
                                      &err);
@@ -400,40 +423,33 @@ meta_prefs_init (void)
   cleanup_error (&err);
   update_num_workspaces (int_val);
 
-  bool_val = gconf_client_get_bool (default_client, KEY_APPLICATION_BASED,
-                                    &err);
-  cleanup_error (&err);
-  update_application_based (bool_val);
+  if (get_bool (KEY_APPLICATION_BASED, &bool_val))
+    update_application_based (bool_val);
 
-  bool_val = gconf_client_get_bool (default_client, KEY_DISABLE_WORKAROUNDS,
-                                    &err);
-  cleanup_error (&err);
-  update_disable_workarounds (bool_val);
+  if (get_bool (KEY_DISABLE_WORKAROUNDS, &bool_val))
+    update_disable_workarounds (bool_val);
 
   str_val = gconf_client_get_string (default_client, KEY_BUTTON_LAYOUT,
                                      &err);
   cleanup_error (&err);
   update_button_layout (str_val);
   g_free (str_val);
-  
-  bool_val = gconf_client_get_bool (default_client, KEY_VISUAL_BELL,
-                                    &err);
-  cleanup_error (&err);
-  bool_val_2 = gconf_client_get_bool (default_client, KEY_AUDIBLE_BELL,
-                                    &err);
-  cleanup_error (&err);
-  update_visual_bell (bool_val, bool_val_2);
 
+  bool_val = provide_visual_bell;
+  bool_val_2 = bell_is_audible;
+  gboolean update_visual = get_bool (KEY_VISUAL_BELL,  &bool_val);
+  gboolean update_audible = get_bool (KEY_AUDIBLE_BELL, &bool_val_2);
+  if (update_visual || update_audible)
+    update_visual_bell (bool_val, bool_val_2);
+      
   str_val = gconf_client_get_string (default_client, KEY_VISUAL_BELL_TYPE,
                                      &err);
   cleanup_error (&err);
   update_visual_bell_type (str_val);
   g_free (str_val);
 
-  bool_val = gconf_client_get_bool (default_client, KEY_REDUCED_RESOURCES,
-				    &err);
-  cleanup_error (&err);
-  update_reduced_resources (bool_val);
+  if (get_bool (KEY_REDUCED_RESOURCES, &bool_val))
+    update_reduced_resources (bool_val);
 
   str_val = gconf_client_get_string (default_client, KEY_TERMINAL_COMMAND,
                                      &err);
@@ -441,11 +457,8 @@ meta_prefs_init (void)
   update_terminal_command (str_val);
   g_free (str_val);
 
-  bool_val = gconf_client_get_bool (default_client, KEY_GNOME_ACCESSIBILITY,
-                                    &err);
-  
-  cleanup_error (&err);
-  update_gnome_accessibility (bool_val);
+  if (get_bool (KEY_GNOME_ACCESSIBILITY, &bool_val))
+    update_gnome_accessibility (bool_val);
 
   str_val = gconf_client_get_string (default_client, KEY_CURSOR_THEME,
                                      &err);
@@ -457,6 +470,14 @@ meta_prefs_init (void)
                                   &err);
   cleanup_error (&err);
   update_cursor_size (int_val);
+#else  /* HAVE_GCONF */
+  /* Set defaults for some values that can't be set at initialization time of
+   * the static globals.  In the case of the theme, note that there is code
+   * elsewhere that will do everything possible to fallback to an existing theme
+   * if the one here does not exist.
+   */
+  titlebar_font = pango_font_description_from_string ("Sans Bold 10");
+  current_theme = g_strdup ("Atlanta");
 #endif /* HAVE_GCONF */
   
   /* Load keybindings prefs */
@@ -1109,7 +1130,11 @@ update_cursor_size (int value)
 {
   int old = cursor_size;
 
-  cursor_size = value;
+  if (value >= 1 && value <= 128)
+    cursor_size = value;
+  else
+    meta_warning (_("%d stored in GConf key %s is not a reasonable cursor_size; must be in the range 1..128\n"),
+                    value, KEY_CURSOR_SIZE);
 
   return old != value;
 }
@@ -1159,7 +1184,6 @@ update_visual_bell_type (const char *value)
 
   return (visual_bell_type != old_bell_type);
 }
-#endif /* HAVE_GCONF */
 
 static gboolean
 update_visual_bell (gboolean visual_bell, gboolean audible_bell)
@@ -1175,6 +1199,7 @@ update_visual_bell (gboolean visual_bell, gboolean audible_bell)
 
   return has_changed;
 }
+#endif /* HAVE_GCONF */
 
 #ifdef HAVE_GCONF
 static gboolean
@@ -1455,6 +1480,10 @@ action_double_click_titlebar_from_string (const char *str)
     return META_ACTION_DOUBLE_CLICK_TITLEBAR_TOGGLE_SHADE;
   else if (strcmp (str, "toggle_maximize") == 0)
     return META_ACTION_DOUBLE_CLICK_TITLEBAR_TOGGLE_MAXIMIZE;
+  else if (strcmp (str, "minimize") == 0)
+    return META_ACTION_DOUBLE_CLICK_TITLEBAR_MINIMIZE;
+  else if (strcmp (str, "none") == 0)
+    return META_ACTION_DOUBLE_CLICK_TITLEBAR_NONE;
   else
     return META_ACTION_DOUBLE_CLICK_TITLEBAR_LAST;
 }
@@ -1583,36 +1612,30 @@ meta_preference_to_string (MetaPreference pref)
 
     case META_PREF_BUTTON_LAYOUT:
       return "BUTTON_LAYOUT";
-      break;
 
     case META_PREF_WORKSPACE_NAMES:
       return "WORKSPACE_NAMES";
-      break;
 
     case META_PREF_VISUAL_BELL:
       return "VISUAL_BELL";
-      break;
 
     case META_PREF_AUDIBLE_BELL:
       return "AUDIBLE_BELL";
-      break;
 
     case META_PREF_VISUAL_BELL_TYPE:
       return "VISUAL_BELL_TYPE";
-      break;
 
     case META_PREF_REDUCED_RESOURCES:
       return "REDUCED_RESOURCES";
-      break;
+
     case META_PREF_GNOME_ACCESSIBILITY:
       return "GNOME_ACCESSIBILTY";
-      break;
+
     case META_PREF_CURSOR_THEME:
       return "CURSOR_THEME";
-      break;
+
     case META_PREF_CURSOR_SIZE:
       return "CURSOR_SIZE";
-      break;
     }
 
   return "(unknown)";
@@ -1753,6 +1776,52 @@ static MetaKeyPref window_bindings[] = {
   { NULL, 0, 0, FALSE }
 };
 
+#ifndef HAVE_GCONF
+typedef struct
+{
+  const char *name;
+  const char *keybinding;
+} MetaSimpleKeyMapping;
+
+/* Name field must occur in the same order as screen_bindings, though entries
+ * can be skipped
+ */
+static MetaSimpleKeyMapping screen_string_bindings[] = {
+  { META_KEYBINDING_WORKSPACE_LEFT,         "<Control><Alt>Left"         },
+  { META_KEYBINDING_WORKSPACE_RIGHT,        "<Control><Alt>Right"        },
+  { META_KEYBINDING_WORKSPACE_UP,           "<Control><Alt>Up"           },
+  { META_KEYBINDING_WORKSPACE_DOWN,         "<Control><Alt>Down"         },
+  { META_KEYBINDING_SWITCH_WINDOWS,         "<Alt>Tab"                   },
+  { META_KEYBINDING_SWITCH_PANELS,          "<Control><Alt>Tab"          },
+  { META_KEYBINDING_CYCLE_WINDOWS,          "<Alt>Escape"                },
+  { META_KEYBINDING_CYCLE_PANELS,           "<Control><Alt>Escape"       },
+  { META_KEYBINDING_SHOW_DESKTOP,           "<Control><Alt>d"            },
+  { META_KEYBINDING_PANEL_MAIN_MENU,        "<Alt>F1"                    },
+  { META_KEYBINDING_PANEL_RUN_DIALOG,       "<Alt>F2"                    },
+  { META_KEYBINDING_COMMAND_SCREENSHOT,     "Print"                      },
+  { META_KEYBINDING_COMMAND_WIN_SCREENSHOT, "<Alt>Print"                 },
+  { NULL,                                   NULL                         }
+};
+
+/* Name field must occur in the same order as window_bindings, though entries
+ * can be skipped
+ */
+static MetaSimpleKeyMapping window_string_bindings[] = {
+  { META_KEYBINDING_WINDOW_MENU,            "<Alt>Print"                 },
+  { META_KEYBINDING_MAXIMIZE,               "<Alt>F10"                   },
+  { META_KEYBINDING_UNMAXIMIZE,             "<Alt>F5"                    },
+  { META_KEYBINDING_MINIMIZE,               "<Alt>F9"                    },
+  { META_KEYBINDING_CLOSE,                  "<Alt>F4"                    },
+  { META_KEYBINDING_BEGIN_MOVE,             "<Alt>F7"                    },
+  { META_KEYBINDING_BEGIN_RESIZE,           "<Alt>F8"                    },
+  { META_KEYBINDING_MOVE_WORKSPACE_LEFT,    "<Control><Shift><Alt>Left"  },
+  { META_KEYBINDING_MOVE_WORKSPACE_RIGHT,   "<Control><Shift><Alt>Right" },
+  { META_KEYBINDING_MOVE_WORKSPACE_UP,      "<Control><Shift><Alt>Up"    },
+  { META_KEYBINDING_MOVE_WORKSPACE_DOWN,    "<Control><Shift><Alt>Down"  },
+  { NULL,                                   NULL                         }
+};
+#endif /* NOT HAVE_GCONF */
+
 static void
 init_bindings (void)
 {
@@ -1799,6 +1868,42 @@ init_bindings (void)
       g_free (str_val);      
       g_free (key);
 
+      ++i;
+    }
+#else /* HAVE_GCONF */
+  int i = 0;
+  int which = 0;
+  while (window_string_bindings[i].name)
+    {
+      /* Find which window_bindings entry this window_string_bindings entry
+       * corresponds to.
+       */
+      while (strcmp(window_bindings[which].name, 
+                    window_string_bindings[i].name) != 0)
+        which++;
+
+      /* Set the binding */
+      update_binding (&window_bindings[which],
+                      window_string_bindings[i].keybinding);
+
+      ++i;
+    }
+
+  i = 0;
+  which = 0;
+  while (screen_string_bindings[i].name)
+    {
+      /* Find which window_bindings entry this window_string_bindings entry
+       * corresponds to.
+       */
+      while (strcmp(screen_bindings[which].name, 
+                    screen_string_bindings[i].name) != 0)
+        which++;
+
+      /* Set the binding */
+      update_binding (&screen_bindings[which], 
+                      screen_string_bindings[i].keybinding);
+
       ++i;
     }
 #endif /* HAVE_GCONF */
@@ -1830,6 +1935,10 @@ init_commands (void)
 
       ++i;
     }
+#else
+  int i;
+  for (i = 0; i < MAX_COMMANDS; i++)
+    commands[i] = NULL;
 #endif /* HAVE_GCONF */
 }
 
@@ -1861,10 +1970,16 @@ init_workspace_names (void)
 
       ++i;
     }
+#else
+  int i;
+  for (i = 0; i < MAX_REASONABLE_WORKSPACES; i++)
+    workspace_names[i] = g_strdup_printf (_("Workspace %d"), i + 1);
+
+  meta_topic (META_DEBUG_PREFS,
+              "Initialized workspace names\n");
 #endif /* HAVE_GCONF */
 }
 
-#ifdef HAVE_GCONF
 static gboolean
 update_binding (MetaKeyPref *binding,
                 const char  *value)
@@ -1912,6 +2027,7 @@ update_binding (MetaKeyPref *binding,
   return changed;
 }
 
+#ifdef HAVE_GCONF
 static const gchar*
 relative_key (const gchar* key)
 {
@@ -2246,6 +2362,9 @@ meta_prefs_change_workspace_name (int         i,
     }
   
   g_free (key);
+#else
+  g_free (workspace_names[i]);
+  workspace_names[i] = g_strdup (name);
 #endif /* HAVE_GCONF */
 }
 
